@@ -3,11 +3,12 @@
 
 import copy
 import warnings
-from collections.abc import Callable
+from collections.abc import Callable, Iterator
 from functools import partial
 from typing import Any
 from uuid import uuid4
 
+from anyio import lowlevel
 from pycrdt import Array, Awareness, Doc, Map, Text
 
 from .utils import cast_all
@@ -112,6 +113,7 @@ class YNotebook(YBaseDoc):
         if meta is None:
             meta = self._ymeta.to_py()
         cell = ycell.to_py()
+        assert cell is not None
         cell.pop("execution_state", None)
         cast_all(cell, float, int)  # cells coming from Yjs have e.g. execution_count as float
         if (
@@ -209,12 +211,27 @@ class YNotebook(YBaseDoc):
         :return: Document's content.
         :rtype: Dict
         """
+        # YNotebook._get is a generator that yields while processing cells.
+        # It is used in YNotebook.get and YNotebook.aget, the former just
+        # runs it until completion and the latter inserts async checkpoints
+        # in order to not block the event loop for too long for notebooks
+        # with a lot of cells.
+        for val in self._get(deduplicate):
+            pass
+
+        assert val is not None
+        return val
+
+    def _get(self, deduplicate: bool) -> Iterator[dict | None]:
         meta = self._ymeta.to_py()
         cast_all(meta, float, int)  # notebook coming from Yjs has e.g. nbformat as float
         cells = []
-        seen_ids = {}  # maps cell_id -> (index, cell converted to Python dict)
+        seen_ids: dict[
+            str, tuple[int, dict]
+        ] = {}  # maps cell_id -> (index, cell converted to Python dict)
 
         for i in range(len(self._ycells)):
+            yield None
             cell = self._cell_to_py(self._ycells[i], meta)
             cell_id = cell.get("id")
 
@@ -254,7 +271,7 @@ class YNotebook(YBaseDoc):
 
             cells.append(cell)
 
-        return dict(
+        yield dict(
             cells=cells,
             metadata=meta.get("metadata", {}),
             nbformat=int(meta.get("nbformat", 0)),
@@ -268,6 +285,15 @@ class YNotebook(YBaseDoc):
         :param value: The content of the document.
         :type value: Dict
         """
+        # YNotebook._set is a generator that yields while processing cells.
+        # It is used in YNotebook.set and YNotebook.aset, the former just
+        # runs it until completion and the latter inserts async checkpoints
+        # in order to not block the event loop for too long for notebooks
+        # with a lot of cells.
+        for _ in self._set(value):
+            pass
+
+    def _set(self, value: dict) -> Iterator[None]:
         nb_without_cells = {key: value[key] for key in value.keys() if key != "cells"}
         nb = copy.deepcopy(nb_without_cells)
         cast_all(nb, int, float)  # Yjs expects numbers to be floating numbers
@@ -289,6 +315,7 @@ class YNotebook(YBaseDoc):
         # to handle the case where the stored doc already has duplicate IDs.
         old_ycells_by_id: dict[str, Map] = {}
         for ycell in self._ycells:
+            yield
             cell_id = ycell.get("id")
             if cell_id is not None and cell_id not in old_ycells_by_id:
                 old_ycells_by_id[cell_id] = ycell
@@ -299,6 +326,7 @@ class YNotebook(YBaseDoc):
 
             # Determine cells to be retained
             for new_cell in new_cells:
+                yield
                 cell_id = new_cell.get("id")
                 if cell_id and (old_ycell := old_ycells_by_id.get(cell_id)):
                     old_cell = self._cell_to_py(old_ycell, meta)
@@ -320,7 +348,11 @@ class YNotebook(YBaseDoc):
             else:
                 index = 0
                 seen: set[str] = set()
-                for old_ycell in list(self._ycells):
+                while True:
+                    yield
+                    if index == len(self._ycells):
+                        break
+                    old_ycell = self._ycells[index]
                     cell_id = old_ycell.get("id")
                     if cell_id is None or cell_id not in retained_cells or cell_id in seen:
                         self._ycells.pop(index)
@@ -330,6 +362,7 @@ class YNotebook(YBaseDoc):
 
             # Now reorder/insert cells to match new_cell_list
             for index, new_cell in enumerate(new_cell_list):
+                yield
                 new_id = new_cell.get("id")
 
                 # Fast path: correct cell already at this position
@@ -340,6 +373,7 @@ class YNotebook(YBaseDoc):
                 if new_id is not None and new_id in retained_cells:
                     # Linear scan to find the cell (O(n) per retained cell)
                     for cur in range(index + 1, len(self._ycells)):
+                        yield
                         if self._ycells[cur].get("id") == new_id:
                             # Use delete+recreate instead of move() for yjs 13.x compatibility
                             # (yjs 13.x doesn't support the move operation that pycrdt generates)
@@ -376,6 +410,33 @@ class YNotebook(YBaseDoc):
                 metadata.setdefault("kernelspec", {"name": "", "display_name": ""})
                 self._ymeta["metadata"] = Map(metadata)
 
+    async def aget(self, deduplicate: bool = True) -> dict:
+        """
+        Returns the content of the document, yielding to the event loop often enough
+        to not block it for too long.
+
+        :param deduplicate: Whether to deduplicate cells with the same ID.
+        :type deduplicate: bool
+        :return: Document's content.
+        :rtype: Dict
+        """
+        for val in self._get(deduplicate):
+            await lowlevel.checkpoint()
+
+        assert val is not None
+        return val
+
+    async def aset(self, value: dict) -> None:
+        """
+        Sets the content of the document, yielding to the event loop often enough
+        to not block it for too long.
+
+        :param value: The content of the document.
+        :type value: Dict
+        """
+        for val in self._set(value):
+            await lowlevel.checkpoint()
+
     def observe(self, callback: Callable[[str, Any], None]) -> None:
         """
         Subscribes to document changes.
@@ -420,17 +481,17 @@ class YNotebook(YBaseDoc):
                         return False
 
                     if kind == Text:
-                        old: Text = old_ycell[key]
-                        old.clear()
-                        old += value
+                        old_text: Text = old_ycell[key]
+                        old_text.clear()
+                        old_text += value
                     elif kind == Array:
-                        old: Array = old_ycell[key]
-                        old.clear()
-                        old.extend(value)
+                        old_array: Array = old_ycell[key]
+                        old_array.clear()
+                        old_array.extend(value)
                     elif kind == Map:
-                        old: Map = old_ycell[key]
-                        old.clear()
-                        old.update(value)
+                        old_map: Map = old_ycell[key]
+                        old_map.clear()
+                        old_map.update(value)
                 else:
                     old_ycell[key] = new_cell[key]
 
